import os
import os.path as osp
import shutil
from pathlib import Path

import typer
from loguru import logger
from pyfzf.pyfzf import FzfPrompt


def cp_jam_template(
    is_debug: bool = typer.Option(
        False, "--v", help="only show content not direct copy"
    )
):
    """
    select and copy jam template files into pwd, files saved in `jamexp/template_data`
    """
    jamexp_path = Path(__file__).parent.parent
    jam_template_path = osp.join(jamexp_path, "template_data")
    whole_template = {
        item: osp.join(jam_template_path, item)
        for item in os.listdir(jam_template_path)
    }
    try:
        fzf = FzfPrompt()
        select = fzf.prompt(whole_template.keys(), "-m")
    except Exception as error:  # pylint: disable=broad-except
        raise RuntimeError(  # pylint: disable=raise-missing-from
            "FZF error: {}".format(error)
        )
    if is_debug:
        for item in select:
            logger.info(item)
            logger.info(f"cp {whole_template[item]} {osp.join(os.getcwd(), item)}")
            with open(whole_template[item], "r", encoding="utf8") as fp:
                for _ in range(5):
                    print(fp.readline())
    else:
        for item in select:
            shutil.copy(whole_template[item], osp.join(os.getcwd(), item))


def cp_template():
    typer.run(cp_jam_template)


if __name__ == "__main__":
    cp_jam_template(is_debug=True)



import os
import subprocess
from signal import SIGKILL

import gpustat
import typer
from loguru import logger
from pyfzf.pyfzf import FzfPrompt


def get_pname(pid):
    with subprocess.Popen(
        ["ps -o cmd= {}".format(pid)], stdout=subprocess.PIPE, shell=True
    ) as proc:
        return str(proc.communicate()[0])


def get_gpu_proc_info():
    query = gpustat.new_query()
    proc_info = {}
    for i_th, cur_gpu in enumerate(query):
        if len(cur_gpu.processes) > 0:
            pinfos = [
                {
                    "pid": cur_p["pid"],
                    "pname": get_pname(cur_p["pid"]),
                    "user": cur_p["username"],
                    "mem": cur_p["gpu_memory_usage"],
                }
                for cur_p in cur_gpu.processes
            ]
            proc_info[i_th] = pinfos
    return proc_info


def kill_all_proc(
    is_debug: bool = typer.Option(False, "--v", help="only show info, not kill proc"),
):
    proc_info = get_gpu_proc_info()
    for i_th, cur_all_pinfo in proc_info.items():
        for cur_pinfo in cur_all_pinfo:
            logger.info(
                f"GPU {i_th:02d}  {cur_pinfo['user']: <10}   {cur_pinfo['pname']}"
            )

    for i_th, cur_all_pinfo in proc_info.items():
        for cur_pinfo in cur_all_pinfo:
            if cur_pinfo["user"] == os.getlogin():
                if not is_debug:
                    os.kill(cur_pinfo["pid"], SIGKILL)
                logger.warning(f"Kill {cur_pinfo['pname']}")


def fzf_kill_proc(
    is_debug: bool = typer.Option(False, "--v", help="only show info, not kill proc"),
):
    proc_info = get_gpu_proc_info()
    info = {}
    for i_th, cur_all_pinfo in proc_info.items():
        for cur_pinfo in cur_all_pinfo:
            if cur_pinfo["user"] == os.getlogin():
                info[f"GPU {i_th:02d} {cur_pinfo['pname']}"] = cur_pinfo["pid"]
    try:
        fzf = FzfPrompt()
        select = fzf.prompt(list(info.keys()), "-m")
    except Exception as error:  # pylint: disable=broad-except
        raise RuntimeError(  # pylint: disable=raise-missing-from
            "FZF error: {}".format(error)
        )
    for cur_p in select:
        logger.warning(f"KILL   {cur_p}")
    if not is_debug:
        for cur_p in select:
            os.kill(info[cur_p], SIGKILL)


def cudakill():
    typer.run(kill_all_proc)


def cudakillfzf():
    typer.run(fzf_kill_proc)



from datetime import datetime

import typer

from jamexp.wandb_clean import get_proj_runs


def check(
    proj: str,
    thres: float = typer.Argument(100, help="threshold for device dead runs~(min)"),
    is_all: bool = typer.Option(False, "--a", help="list all running"),
):
    """
    check running runs in wandb project, print how long it does not update, its create time, and last update time
    """
    rtn_run = []
    for run in get_proj_runs(proj):
        if run.state == "running":
            url = run.url
            run.create
            last_update_time = datetime.fromtimestamp(run.summaryMetrics["_timestamp"])
            create_time = datetime.strptime(run.createdAt, "%Y-%m-%dT%H:%M:%S")
            now_time = datetime.now()
            timegap_lastupdate_now = (now_time - last_update_time).seconds // 60
            if is_all or timegap_lastupdate_now > thres:
                print(
                    f"{run.name:<50}\n \t{timegap_lastupdate_now} min! Create! {create_time} Last! {last_update_time} {url}"
                )
                rtn_run.append(run)
    return rtn_run


def main():
    typer.run(check)


if __name__ == "__main__":
    main()



import json
import os
from collections import defaultdict
from datetime import datetime, timedelta, timezone

import typer
from loguru import logger
from plumbum.commands.processes import ProcessExecutionError
from pyfzf.pyfzf import FzfPrompt

from jamexp.utils.os_shell import run_simple_command


def get_job_info(team, key):
    team = 'lpr-imagine' if team else 'deep-imagination'

    stdout, stderr = run_simple_command(f'ngc batch list --format_type json --team {team}')
    jobs_info = defaultdict(list)
    latest_run, latest_td = None, timedelta(365)
    if stderr:
        print('ngc batch list error')
        print(stderr)
    else:
        list_job = json.loads(stdout)
        for job in list_job:
            status = job['jobStatus']['status']
            name = job['jobDefinition']['name']
            job_id = job['id']
            if key in name:
                if status in ['STARTING', 'RUNNING', 'QUEUED']:
                    if status == 'RUNNING':
                        s_t = datetime.strptime(job['jobStatus']['startedAt'], '%Y-%m-%dT%H:%M:%S.000Z')
                        cur_t = datetime.now(timezone.utc).replace(tzinfo=None)
                        td = (cur_t - s_t)
                        _info = f"{job_id:<15} {name:<35} {status:<10} {td.days}D {td.seconds//3600}H {(td.seconds//60)%60}M"
                        jobs_info[status].append(_info)
                        if td < latest_td:
                            latest_run = _info
                            latest_td = td
                    elif status == 'QUEUED':
                        s_t = datetime.strptime(job['jobStatus']['queuedAt'], '%Y-%m-%dT%H:%M:%S.000Z')
                        cur_t = datetime.now(timezone.utc).replace(tzinfo=None)
                        td = (cur_t - s_t)
                        jobs_info[status].append(f"{job_id:<15} {name:<35} {status:<10} {td.days}D {td.seconds//3600}H {(td.seconds//60)%60}M")
                    else:
                        jobs_info[status].append(f"{job_id:<15} {name:<35} {status:<10}")
    return jobs_info, latest_run

def _ngcbash(
    team : bool = typer.Option(True, "-t/-T", help="defualt use lpr-imagine, otherwise use deep-imagination"),
    latest : bool = typer.Option(True, "-l/-L", help="defualt exec latest job bash, otherwise use fzf"),
    key : str = typer.Argument('ml', help='filter key, only show jobs whose names contain the key'),
):
    jobs_info,latest_run = get_job_info(team, key)
    if latest:
        if latest_run:
            select_run = latest_run
        else:
            exit(0)
    else:
        try:
            fzf = FzfPrompt()
            select_run = fzf.prompt(jobs_info['RUNNING'])
        except ProcessExecutionError:
            exit(1)
        except Exception as error:  # pylint: disable=broad-except
            raise RuntimeError(  # pylint: disable=raise-missing-from
                "FZF error: {}".format(error)
            )

    logger.warning(f"bash exec {select_run[0]}")
    job_id = select_run[0][:7]
    os.system(f'ngc batch exec {job_id}')

def _ngckill(
    team : bool = typer.Option(True, "-t/-T", help="defualt use lpr-imagine, otherwise use deep-imagination"),
    live : bool = typer.Option(False, "-l/-L", help="live run or dryrun, default dryrun"),
    kill_all : bool = typer.Option(False, "-a/-A", help="kill all jobs or not, default not"),
    key : str = typer.Argument('ml', help='filter key, only show jobs whose names contain the key')
):
    jobs_info = get_job_info(team, key)[0]

    _keys = []
    for k,v in jobs_info.items():
        _keys.extend([f'{k:<10}{_v}' for _v in v])

    if kill_all:
        for cur_line in _keys:
            _job_id = cur_line[10:10+7]
            run_simple_command(f"ngc batch kill {_job_id}")
            logger.warning(f"KILL   {cur_line}")
    elif live:
        try:
            fzf = FzfPrompt()
            select = fzf.prompt(_keys, "-m")
        except ProcessExecutionError:
            exit(1)
        except Exception as error:  # pylint: disable=broad-except
            raise RuntimeError(  # pylint: disable=raise-missing-from
                "FZF error: {}".format(error)
            )
        for cur_line in select:
            _job_id = cur_line[10:10+7]
            run_simple_command(f"ngc batch kill {_job_id}")
            logger.warning(f"KILL   {cur_line}")

    if not live:
        for k,v in jobs_info.items():
            print("#"*10 + f" {team}  {k}  Total {len(v):<10}" + "#"*10)
            for item in v:
                print(item)

def ngc_kill():
    typer.run(_ngckill)


def ngc_bash():
    typer.run(_ngcbash)



import typer


def helper():
    cmds = {
        "jcpt": "copy template file from jamexp",
        "jkill": "kill processes using cuda, selected by FZF",
        "jkilla": "kill all processes using cuda",
        "jln": "ln data dir",
        "jjnln": "delte link",
        "jwb_check_running": "check wandb running status",
        "jnb_clean": "clean notebook",
    }
    for key, value in cmds.items():
        print(f"{key:<20}\t\t {value}")


def run_helper():
    typer.run(helper)



import typer


def jnb_clean(
    folder: str = typer.Argument(
        "notebooks", help="threshold for device dead runs~(min)"
    ),
    is_debug: bool = typer.Option(False, "--v", help="only show info not run"),
):
    """
    clean notebooks folder, only git clean data, but keep src for reproduce. skip nb file startswith skip
    """
    try:
        import jammy.io as jio
        from jammy.io.path import glob
        from jammy.utils.git import git_rootdir
        from jammy.utils.process import run_simple_command
    except ImportError:
        print("jammy needed to run the func")
        exit(1)

    for in_nb_file in glob(git_rootdir(folder), regex="ipynb$"):
        if ".ipynb_checkpoints" in in_nb_file:
            continue
        names = in_nb_file.split("/")
        nb_file_name = names[-1].split(".")[0]
        if nb_file_name.startswith("_clean_") or nb_file_name.startswith("skip"):
            continue
        out_nb_file = "/".join(names[:-1]) + f"/_clean_{nb_file_name}.ipynb"
        jio.copy(in_nb_file, out_nb_file)
        cmd = f"cat {in_nb_file} | nbstripout > {out_nb_file}"
        if is_debug:
            print(cmd)
        else:
            run_simple_command(cmd)


def main():
    typer.run(jnb_clean)
